@@ -3,7 +3,7 @@ package Mail::Bulkmail::Object;
 #Copyright and (c) 1999, 2000, 2001, 2002, 2003 James A Thomason III (jim@jimandkoka.com). All rights reserved.
 #The Mail::Bulkmail::Object component may only be distributed in conjunction with Mail::Bulkmail.
 #Mail::Bulkmail::Object is licensed under the artistic license for the creation classes related to Mail::Bulkmail, or
-#to enable the use and running of those classes. 
+#to enable the use and running of those classes.
 
 # SCROLL DOWN TO @conf_files ARRAY TO CONFIGURE IT
 
@@ -27,7 +27,7 @@ was born.
 
 Of course, you don't have to use this to create subclasses, but you'll run the risk of making something with an inconsistent
 interface vs. the rest of the system. That'll confuse people and make them unhappy. So I recommend subclassing off of here
-to be consistent. Of course, you may not like these objects, but they do work well and are consistent. Consistency is 
+to be consistent. Of course, you may not like these objects, but they do work well and are consistent. Consistency is
 very important in interface design, IMHO.
 
 The license is a little more restrictive. Basically, you can use it for anything you'd like as long as it's related to
@@ -36,16 +36,16 @@ Go to town. Use it under the artistic license for Mail::Bulkmail use, distribute
 
 =cut
 
-$VERSION = '3.05';
+$VERSION = '3.06';
 
 use Socket;
 use 5.6.0;
-use Data::Dumper ();
+#use Data::Dumper ();
 
-sub dump {
-	my $self = shift;
-	return Data::Dumper::Dumper($self);
-};
+#sub dump {
+#	my $self = shift;
+#	return Data::Dumper::Dumper($self);
+#};
 
 use strict;
 use warnings;
@@ -60,14 +60,14 @@ You'll need to specify your conf files. There is the @conf_files array, toss in
  	/etc/mail.bulkmail.cfg
  	/etc/mail.bulkmail.cf2
  );
- 
+
 It'll just silently ignore any conf files that aren't present, so don't expect any errors. That's to allow you
 to place multiple conf files in for use on multiple servers and then not worry about them.
 
 Multiple conf files are in significance order. So if mail.bulkmail.cfg and mail.bulkmail.cf2 both define a value
 for 'foo', then the one in mail.bulkmail.cfg is used. And so on, conf files listed earlier are more important.
 There is no way for a program to later look at a less significant conf value.
- 
+
 =cut
 
 #you'll need to specify your conf files
@@ -87,15 +87,15 @@ There is no way for a program to later look at a less significant conf value.
 conf_files returns your conf_files array.
 
  my @conf_files = $class->conf_files();
- 
+
 You can also programmatically add a new conf_file this way.
 
  $class->conf_files('/path/to/new/conf.file', '/path/to/other/conf.file');	#, etc
- 
+
 However, it'd be better to specify your conf file at use time.
 
  use Mail::Bulkmail::Object 3.00 "/path/to/conf.file";
- 
+
 This also (naturally) works in all subclasses.
 
  use Mail::Bulkmail 3.00 "/path/to/conf.file";
@@ -109,11 +109,11 @@ i.e., those conf files are more significant.
 So,
 
  @conf_files = qw(/path/to/file /path/to/file2);
- 
+
  use Mail::Bulkmail::Object 3.00 "/path/to/file3" "/path/to/file4";
- 
+
  Mail::Bulkmail::Object->conf_files("/path/to/file5", "/path/to/file6");
- 
+
  print Mail::Bulkmail::Object->conf_files;
  	#prints out /path/to/file5 /path/to/file6 /path/to/file3 /path/to/file4 /path/to/file path/to/file2
 
@@ -121,21 +121,21 @@ Note that you don't *need* conf files, you can still specify all information at
 via mutators, or whatever. But a conf file can make your life a lot easier.
 
 =cut
-	
+
 	sub conf_files {
 		my $self = shift;
 		unshift @conf_files, $_ foreach reverse @_;
 		return @conf_files;
 	};
-	
+
 	# the importer looks to any arguments specified at import and puts them
 	# on the FRONT of the conf_files array.
 	sub import {
 		my $class = shift;
 		unshift @conf_files, $_ foreach reverse @_;
 		return 1;
-	};	
-	
+	};
+
 };
 
 # You really probably don't want to change this
@@ -161,17 +161,17 @@ of course, a blessed ((thingie)) (scalar, array, code, hash, etc) reference. And
 things, depending upon the situation. Hashes are easy to work with and most similar to traditional objects.
 
  $object->{$attribute} = $value;
- 
+
 And whatnot. Arrays are much faster (typically 33% in tests I've done), but they suck to work with.
 
  $object->[15] = $value;	#the hell is '15'?
- 
+
  (
   by the way, you can make this easier with variables defined to return the value, i.e.
   $object->[$attribute] = $value;	#assuming $attribute == 15
  )
- 
-Scalars are speciality and coderefs are left to the magicians. Don't get me wrong, coderefs as objects are nifty, but 
+
+Scalars are speciality and coderefs are left to the magicians. Don't get me wrong, coderefs as objects are nifty, but
 they can be tricky to work with.
 
 So, I wanted a consistent interface. I'm not going to claim credit for this idea, since I think I originally read it
@@ -181,7 +181,7 @@ originally detailed in there. Anyway, I liked it a lot and decided I'd implement
 Basically, attributes are accessed and mutated via methods.
 
  $object->attribute($value);
- 
+
 For all attributes. This way, the internal object can be whatever you'd like. I used to use mainly arrays for the speed
 boost, but lately I use hashes a lot because of the ease of dumping and reading the structure for debugging purposes.
 But, with this consistent interface of using methods to wrapper the attributes, I can change the implementation of
@@ -200,7 +200,7 @@ negligible. And you can't do nice things like:
  $object->{$attribute}++;
  you'd have to do
  $object->attribute($object->attribute + 1);
- 
+
 Which is annoying. But I think it's offset by the consistent interface regardless of what your underlying object is.
 
 Enough with the philosophy, though. You need to know how this works.
@@ -229,7 +229,7 @@ to the internal _accessor method which handles the mutating and accessing.
 There is another syntax for add_attr, to define a different internal accessor:
 
  Some::Class->add_attr(['foo', 'other_accessor']);
- 
+
 This creates method called 'foo' which talks to a separate accessor, in this case "other_accessor" instead of going
 to _accessor. This is useful if you want to create a validating method on your attribute.
 
@@ -241,13 +241,13 @@ slot your other accessor with use. In generall, for a given "attribute", "_attri
 Example:
 
  Some::Class->add_attr(['foo', 'other_accessor']);
- 
+
  $obj->foo('bee');
- 
+
  sub other_accessor {
  	my $self	= shift;
  	my $method	= shift;	# "_foo", in this example
- 	
+
 	if (@_){
 		my $val = shift;	# "bee", in this example
 		if ($val == 7){
@@ -261,24 +261,24 @@ Example:
  		return $self->$method();
  	};
  };
- 
+
 And, finally, you can also pass in additional arguments as static args if desired.
 
  Some::Class->add_attr(['foo', 'other_accessor'], 'bar');
- 
+
  $obj->foo('bee');
- 
+
  sub other_accessor {
  	my $self	= shift;
  	my $method	= shift;
  	my $static 	= shift;	#'bar' in our example
- 	
+
  	my $value	= shift;	#'bee' in our example
  	.
  	.
  	.
  };
- 
+
  All easy enough. Refer to any subclasses of this class for further examples.
 
 =cut
@@ -286,11 +286,11 @@ And, finally, you can also pass in additional arguments as static args if desire
 sub add_attr {
 	my $pkg			= shift;
 	my $method		= shift;
-	
+
 	my $accessor	= "_accessor";
-	
+
 	my @static_args	= @_;
-	
+
 	if (ref $method){
 		($method, $accessor) = @$method;
 		no strict 'refs';
@@ -316,10 +316,10 @@ object and class attributes with the same name. This is by design. (error is a s
 
  Some::Class->add_attr('foo');			#object attribute foo
  Some::Class->add_class_attr('bar'):	#class attribute bar
- 
+
  print $obj->foo();
  print Some::Class->bar();
- 
+
 Behaves the same as an object method added with add_attr, mutating with a value, accessing without one. Note
 that add_class_attr does not have the capability for additional internal methods or static values. If you want
 those on a class method, you'll have to wrapper the class attribute yourself on a per case basis.
@@ -334,7 +334,7 @@ class attributes are automatically initialized to any values in the conf file up
 sub add_class_attr {
 	my $pkg		= shift;
 	my $method	= shift;
-	
+
 	my $f = q{
 			{
 				my $attr = undef;
@@ -348,19 +348,19 @@ sub add_class_attr {
 
 	no strict 'refs';
 	*{$pkg . "::$method"} = eval $f;
-	
+
 	#see if there's anything in the conf file
-	
+
 	my $conf = $pkg->read_conf_file
 		|| die "Conf file error : " . $pkg->error . " " . $pkg->errcode;
 	if ($conf->{$pkg}->{$method}){
 		$pkg->$method($conf->{$pkg}->{$method});
 	};
-	
+
 	if (@_){
 		$pkg->$method(@_);
 	};
-	
+
 	return $method;
 };
 
@@ -376,43 +376,43 @@ into any class as it is called. This is useful for subclasses.
 Watch:
 
  package SuperClass;
- 
+
  SuperClass->add_class_attr('foo');
  SuperClass->foo('bar');
- 
+
  package SubClass;
  @ISA = qw(SuperClass);
- 
+
  print SubClass->foo();			#prints bar
  print SuperClass->foo();		#prints bar
- 
+
  print SuperClass->foo('baz');	#prints baz
  print SubClass->foo();			#prints baz
- 
+
  print SubClass->foo('dee');	#prints dee
  print SuperClass->foo();		#prints dee
- 
+
 See? The attribute is still stored in the super class, so changing it in a subclass changes it in the super class as well.
 Usually, this behavior is fine, but sometimes you don't want that to happen. That's where add_trickle_class_attr comes
-in. Its first call will snag the value from the SuperClass, but then it will have its own attribute that's separate. 
+in. Its first call will snag the value from the SuperClass, but then it will have its own attribute that's separate.
 
 Again, watch:
 
 
  package SuperClass;
- 
+
  SuperClass->add_trickle_class_attr('foo');
  SuperClass->foo('bar');
- 
+
  package SubClass;
  @ISA = qw(SuperClass);
- 
+
  print SubClass->foo();			#prints bar
  print SuperClass->foo();		#prints bar
- 
+
  print SuperClass->foo('baz');	#prints baz
  print SubClass->foo();			#prints bar
- 
+
  print SubClass->foo('dee');	#prints dee
  print SuperClass->foo();		#prints baz
 
@@ -420,15 +420,15 @@ This is useful if you have an attribute that should be unique to a class and all
 
  package SuperClass;
  SuperClass->add_class_attr('foo');
- 
+
  package SubClass
  SubClass->add_class_attr('foo');
- 
+
  and
- 
+
  package SuperClass;
  SuperClass->add_trickle_class_attr('foo');
- 
+
 You'll usually just use add_class_attr. Only use trickle_class_attr if you know you need to, since you rarely would.
 There is a *slight* bit of additional processing required for trickled accessors.
 
@@ -439,7 +439,7 @@ trickled class attributes are automatically initialized to any values in the con
 sub add_trickle_class_attr {
 	my $pkg		= shift;
 	my $method	= shift;
-	
+
 	my $f = qq{
 		{
 			my \$attr = undef;
@@ -459,9 +459,9 @@ sub add_trickle_class_attr {
 					return \$attr;
 				}
 			}
-			
+
 		}
-	};	
+	};
 
 	no strict 'refs';
 	*{$pkg . "::$method"} = eval $f;
@@ -474,17 +474,17 @@ sub add_trickle_class_attr {
 		if ($conf->{$pkg}->{$method}){
 			$pkg->$method($conf->{$pkg}->{$method});
 		};
-	
+
 		if (@_){
 			$pkg->$method(@_);
 		};
 	};
-	
+
 	return $method;
 };
 
 # _accessor is the main accessor method used in the system. It defines the most simple behavior as to how objects are supposed
-# to work. If it's called with no arguments, it returns the value of that attribute. If it's called with arguments, 
+# to work. If it's called with no arguments, it returns the value of that attribute. If it's called with arguments,
 # it sets the object attribute value to the FIRST argument passed and ignores the rest
 #
 # example:
@@ -508,22 +508,22 @@ sub _accessor {
 	return $self->{$prop};
 };
 
-=pod 
+=pod
 
 =item error and errcode
 
 error rocks. All error reporting is set and relayed through error. It's a standard accessor, and an *almost*
-standard mutator. The difference is that when used as a mutator, it returns undef instead of the value
-mutated to.
+standard mutator. The difference is that when used as a mutator, it returns undef (or an empty list) instead
+of the value mutated to.
 
-If a method fails, it is expected to return undef and set error.
+If a method fails, it is expected to return undef (or an empty list) and set error.
 
 example:
 
  sub someMethod {
  	my $self = shift;
  	my $value = shift;
- 	
+
  	if ($value > 10){
  		return 1;		#success
  	}
@@ -531,10 +531,10 @@ example:
  		return $self->error("Values must be greater than 10");
  	};
  };
- 
+
  $object->someMethod(15) || die $object->error;	#succeeds
  $object->someMethod(5)	 || die $object->error;	#dies with an error..."Values must be greater than 10"
- 
+
 Be warned if your method can return '0', this is a valid successful return and shouldn't give an error.
 But most of the time, you're fine with "true is success, false is failure"
 
@@ -549,7 +549,7 @@ example:
  sub someMethod {
  	my $self = shift;
  	my $value = shift;
- 	
+
  	if ($value > 10){
  		return 1;		#success
  	}
@@ -560,7 +560,7 @@ example:
 
  $object->someMethod(15) || die $object->error;		#succeeds
  $object->someMethod(5)	 || die $object->errcode;	#dies with an error code ... "ERR77"
- 
+
 If your code is looking for an error, read the errcode. if a human is looking at it, display the error.
 Easy as pie.
 
@@ -568,14 +568,14 @@ Both classes and objects have error methods.
 
  my $obj = Some::Class->new() || die Some::Class->error();
  $obj->foo() || die $obj->error();
- 
+
 Note that error is a special method, and not just a normal accessor or class attribute. As such:
 
  my $obj = Some::Class->new();
  Some::Class->error('foo');
  print $obj->error();			#prints undef
  print Some::Class->error();	#prints foo
- 
+
 i.e., you will B<not> get a class error message by calling ->error on an object.
 
 There is also an optional third paramenter..."not logged", which sounds horribly ugly, I know. But it is a bit of an
@@ -583,16 +583,19 @@ after-market hack, so it's to be expected. The third argument does what you'd th
 being logged.
 
  $self->error("This is an error message", "code", "not logged");
- 
+
 Any true value may be passed for the 3rd argument, but something that makes it obvious what it's doing is recommended, hence
 my use of 'not logged'. This is useful for bubbling up errors.
 
  $class->error($self->error, $self->errcode, 'not logged');
- 
+
 The reason is that the error was already logged when it was stored in $self. So you'd end up logging it twice in your error
 file, which is very confusing. So it's recommended to use the three argument form for errors that are bubbling up, but not
 elsewhere.
 
+As of 3.06, if an error is returned in a list context, an empty list will be returned instead of undef. undef is still
+returned in a scalar context.
+
 =cut
 
 sub error {
@@ -600,7 +603,7 @@ sub error {
 
 	my $errormethod	= ref $self	? "_obj_error"		: "_pkg_error";
 	my $codemethod	= ref $self	? "_obj_errcode"	: "_pkg_errcode";
-	
+
 	if (@_){
 		my $error	= shift;
 		my $code	= shift;
@@ -608,8 +611,8 @@ sub error {
 		$self->$errormethod($error);
 		$self->$codemethod(defined $code ? $code : undef);
 		$self->logToFile($self->ERRFILE, "error: $error" . (defined $code ? "\tcode : $code" : '')) if !$nolog && $self->ERRFILE && $error;
-	
-		return undef;
+
+		return wantarray ? () : undef;
 	}
 	else {
 		return $self->$errormethod();
@@ -623,16 +626,16 @@ sub error {
 errcode is an accessor ONLY. You can only mutate the errcode via error, see above.
 
  print $obj->errcode;
- 
+
 Both objects and classes have errcode methods.
- 
+
  my $obj = Some::Class->new() || die Some::Class->errcode();
  $obj->foo() || die $obj->errcode();
- 
+
 Where possible, the pod will note errors that a method is known to be able to return. Please
 note that this will B<never> be an all inclusive list of all error codes that may possibly
 ever be returned by this method. Only error codes generated by a particular method will be listed.
- 
+
 =cut
 
 sub errcode {
@@ -655,13 +658,13 @@ Nothing more.
 
 sub errstring {
 	my $self = shift;
-	
-	return 
+
+	return
 		(defined $self->error ? $self->error : '')
-		 . "...with code (" . 
+		 . "...with code (" .
 		 (defined $self->errcode ? $self->errcode : '')
 		 . ")";
-		 
+
 };
 
 =pod
@@ -675,7 +678,7 @@ You can also pass in a list of conf files to read, in most to least significant
  my $conf = Mail::Bulkmail::Object->read_conf_file();
  or
  my $conf = Mail::Bulkmail::Object->read_conf_file('/other/conf.file');
- 
+
 If you pass in a list of conf files, then the internal @conf_files array is bypassed.
 
 $conf is a hashref of hashrefs. the main keys are the package names, the values are the hashes of the values
@@ -685,15 +688,15 @@ Example:
 
  #conf file
  define package Mail::Bulkmail
- 
+
  use_envelope = 1
  Trusting @= duplicates
- 
+
  define package Mail::Bulkmail::Server
- 
+
  Smtp = your.smtp.com
  Port = 25
- 
+
  $conf = {
  	'Mail::Bulkmail' => {
  		'use_envelope' => 1,
@@ -704,7 +707,7 @@ Example:
  		'Port' => 25
  	}
  };
- 
+
 read_conf_file is called at object initialization. Any defaults for your object are read in at this time.
 You'll rarely need to read the conf file yourself, since at object creation it is read and parsed and the values passed
 on.
@@ -722,7 +725,7 @@ this method is known to be able to return MBO002 - Invalid conf file
 	my $loaded		= {};
 	sub read_conf_file {
 		my $class = shift;
-	
+
 		my @confs	= reverse(@_ ? @_ : $class->conf_files());
 		my $conf	= @_ ? {} : $global_conf;
 
@@ -730,7 +733,7 @@ this method is known to be able to return MBO002 - Invalid conf file
 			next unless -e $conf_file ;
 			if (! $loaded->{$conf_file} || -M $conf_file <= 0){
 				my $pkg	 = $default_package;
-				
+
 				open (CONF, $conf_file) || next;
 				while (my $line = <CONF>) {
 					next if ! defined $line || $line =~ /^\s*#/ || $line =~ /^\s*$/;
@@ -748,19 +751,19 @@ this method is known to be able to return MBO002 - Invalid conf file
 					unless (defined $val){
 						($user, $key, $array, $val) = ($user, $key, undef, $array);
 					};
-					
+
 					unless (defined $array){
 						($user, $key, $array, $val) = (undef, $user, $array, $key);
 					};
 
 					($user, $key, $val) = (undef, $user, $key) unless defined $val;
-					
+
 					next if defined $user && $user != $>;
-					
+
 					$val = undef if $val eq 'undef';
 
 					$val = eval qq{return "$val"} if defined $val && $val =~ /^\\/;
-					
+
 					if ($array) {
 						$conf->{$pkg}->{$key} ||= [];
 						push @{$conf->{$pkg}->{$key}}, $val;
@@ -773,7 +776,7 @@ this method is known to be able to return MBO002 - Invalid conf file
 			};	#end if
 		};	#end foreach
 		return $conf;
-		
+
 	};	#end sub
 };
 
@@ -785,11 +788,11 @@ returns a filehandle in a different package. Useful for when you need to open fi
 
  my $handle = Mail::Bulkmail->gen_handle();
  open ($handle, "/path/to/my/list");
- 
+
  my $bulk = Mail::Bulkmail->new(
  	'LIST' => $handle
  );
- 
+
 You never need to use gen_handle if you don't want to. It's used extensively internally, though.
 
 =cut
@@ -800,7 +803,7 @@ You never need to use gen_handle if you don't want to. It's used extensively int
 	sub gen_handle {
 		no strict 'refs';
 		my $self = shift;
-		return \*{"Mail::BulkMail::Handle::HANDLE" . $handle++};	#You'll note that I don't want my 
+		return \*{"Mail::BulkMail::Handle::HANDLE" . $handle++};	#You'll note that I don't want my
 																	#namespace polluted either
 	};
 
@@ -813,7 +816,7 @@ You never need to use gen_handle if you don't want to. It's used extensively int
 Finally! The B<constructor>. It's very easy, for a minimalist object, do this:
 
  my $obj = Class->new() || die Class->error();
- 
+
 Ta da! You have an object. Any attributes specified in the conf file will be loaded into your object. So if your
 conf file defines 'foo' as 'bar', then $obj->foo will now equal 'bar'.
 
@@ -824,17 +827,17 @@ If you'd like, you can also pass in method/value pairs to the constructor.
  	'foo'		=> 'baz',
  	'method'	=> '88'
  ) || die Class->error();
- 
+
 This is (roughly) the same as:
 
  my $obj = Class->new() || die Class->error();
- 
+
  $obj->attribute(17) || die $obj->error();
  $obj->foo('baz') || die $obj->error();
  $obj->method(88) || die $obj->error();
- 
+
 Any accessors or methods you'd like may be passed to the constructor. Any unknown pairs will be silently ignored.
-If you pass a method/value pair to the constructor, it will override any equivalent method/value pair in the 
+If you pass a method/value pair to the constructor, it will override any equivalent method/value pair in the
 conf file.
 
 Additionally, if you need to set up values in your object, this is the place to do it. Note that setting default
@@ -842,7 +845,7 @@ values should probably be done in the conf file, but if you need to populate a d
 
  package SubClass;
  @ISA = qw(SuperClass);
- 
+
  sub new {
  	return shift->new(
  		'servers'		=> [],
@@ -850,7 +853,7 @@ values should probably be done in the conf file, but if you need to populate a d
  		@_
  	);
  };
- 
+
 This will cause your SubClass to use the normal constructor, but get default values of the empty data structures
 specified.
 
@@ -873,7 +876,7 @@ The object initializer. Arguably more important than the constructor, but not so
 The constructor calls it internally, and you really shouldn't touch it or override it. But I wanted it here so
 you know what it does.
 
-Simply, it iterates through the conf file and mutates any of your object attributes to the value specified in the conf 
+Simply, it iterates through the conf file and mutates any of your object attributes to the value specified in the conf
 file. It then iterates through the hash you passed to ->new() and does the same thing, overriding any conf values, if
 necessary.
 
@@ -881,26 +884,27 @@ init is smart enough to use all super class values defined in the conf file, in
 contains:
 
  define package SuperClass
- 
+
  foo = 'bar'
- 
+
 And you're creating a new SubClass object, then it will get the default of foo = 'bar' as in the conf file, despite
 the fact that it was not defined for your own package. Naturally, the more significant definition is used.
 
  define package SuperClass
- 
+
  foo = 'bar'
- 
+
  define package SubClass
- 
+
  foo = 'baz'
 
 SuperClass objects will default foo to 'bar', SubClass objects will default foo to 'baz'
 
-this method is known to be able to return 
+this method is known to be able to return
 
  MBO003 - could not initialize value to conf value
  MBO004 - could not initialize value to constructor value
+ MBO006 - odd number of elements in hash assignment
 
 =cut
 
@@ -908,9 +912,9 @@ sub init {
 	my $self	= shift;
 	my $class	= ref $self;
 
-	my %init	= @_;
+#	my %init	= @_;
 
-	my $conf = $self->read_conf_file 
+	my $conf = $self->read_conf_file
 		|| die "Conf file error : " . $self->error . " " . $self->errcode;
 
 	#initialize our defaults from the conf file
@@ -921,7 +925,7 @@ sub init {
 				$self->errcode(undef);
 				my $return = $self->$method($conf->{$pkg}->{$method}) if $self->can($method);
 				my $value = defined $conf->{$pkg}->{$method} ? $conf->{$pkg}->{$method} : 'value is undef';
-				return $self->error("Could not initilize method ($method) to  value ($value)" 
+				return $self->error("Could not initilize method ($method) to  value ($value)"
 					. (defined $self->error ? " : " . $self->error : '')
 					, ($self->errcode || "MBO003")
 				) unless defined $return;
@@ -930,13 +934,27 @@ sub init {
 	};
 
 	#initialize our defaults as passed in to the constructor
-	foreach my $method (keys %init){
+#	foreach my $method (keys %init){
+
+	while (@_) {
+		my $method	= shift;
+		my $value	= undef;
+
+		if (@_){
+			$value	= shift;
+		}
+		else {
+			return $self->error("Odd number of elements in hash assignment", "MBO006");
+		};
+
 		if ($self->can($method)){
 			$self->error(undef);
 			$self->errcode(undef);
-			my $return = $self->$method($init{$method});
-			my $value = defined $init{$method} ? $init{$method} : 'value is undef';
-			return $self->error("Could not initilize method ($method) to  value ($value)" 
+			#my $return = $self->$method($init{$method});
+			my $return = $self->$method($value);
+			#my $value = defined $init{$method} ? $init{$method} : 'value is undef';
+			my $errval = defined $value ? $value : 'value is undef';
+			return $self->error("Could not initilize method ($method) to  value ($errval)"
 				. (defined $self->error ? " : " . $self->error : '')
 				, ($self->errcode || "MBO004")
 			) unless defined $return;
@@ -960,18 +978,18 @@ Obviously, dynamically changing @ISA is frowned upon as a result.
 
 {
 	my $paths = {};
-	
+
 	sub isa_path {
 		my $class	= shift;
 		my $seen	= shift || {};
-		
+
 		return undef if $seen->{$class}++;
-		
+
 		return $paths->{$class} if $paths->{$class};
-		
+
 		no strict 'refs';
 		my @i = @{$class . "::ISA"};
-	
+
 		my @s = ($class);
 		foreach my $super (@i){
 			next if $seen->{$super};
@@ -979,15 +997,15 @@ Obviously, dynamically changing @ISA is frowned upon as a result.
 			my $super_isa = isa_path($super, $seen);
 			push @s, @$super_isa;
 		};
-		
+
 		@s = reverse @s;	#we want to look at least significant first
-		
+
 		$paths->{$class} = \@s;
-		
+
 		return \@s;
-		
+
 	};
-	
+
 };
 
 # _file_accessor is an internal accessor for accessing external information. Said external information can be in
@@ -1007,7 +1025,7 @@ sub _file_accessor {
 	my $IO		= shift;
 	my $file	= shift;
 
-	if ($file){
+	if (defined $file){
 		if (! ref $file) {
 			my $handle = $self->gen_handle();
 			if ($IO =~ /^(?:>>?|<)$/){
@@ -1031,7 +1049,7 @@ sub _file_accessor {
 	else {
 		return $self->$prop();
 	};
-	
+
 };
 
 =pod
@@ -1041,15 +1059,15 @@ sub _file_accessor {
 getNextLine is called on either a filehandleref, an arrayref, or a coderef
 
  $obj->getNextLine(\*FOO);
- 
+
 will return the next line off of FOO;
 
  $obj->getNextLine(\@foo);
- 
+
 will shift the next line off of @foo and return it.
 
  $obj->getNextLine(\&foo);
- 
+
 will call foo($obj) and return whatever the function returns.
 
 Note that your bulkmail object is the first argument passed to your function. It's not called as a method, but
@@ -1058,21 +1076,21 @@ the object is still the first argument passed.
 This is mainly used with attribues going through _file_accessor.
 
  package SomeClass;
- 
+
  SomeClass->add_attr(['FOO', '_file_accessor'], "<");
  my $obj = SomeClass->new(
  	FOO => \&foo
  ) || die SomeClass->error();
- 
+
  my $val = $obj->getNextLine($obj->FOO);
 
 =cut
 
 sub getNextLine {
 	my $self = shift;
-	
+
 	my $list = shift || $self->LIST() || return $self->error("Cannot get next line w/o list", "MB045");
-	
+
 	if (ref $list eq "GLOB"){
 		my $email = scalar <$list>;
 		return undef unless defined $email;
@@ -1088,7 +1106,7 @@ sub getNextLine {
 	else {
 		return $self->error("Cannot get next line...don't know what a $list is", "MB046");
 	};
-	
+
 };
 
 =pod
@@ -1100,15 +1118,15 @@ logToFile is the opposite of getNextLine, it writes out a value instead of readi
 logToFile is called on either a filehandleref, an arrayref, or a coderef
 
  $obj->logToFile(\*FOO, "bar");
- 
+
 will append a new line to FOO, "bar"
 
  $obj->logToFile(\@foo, "bar");
- 
+
 will push the value "bar" onto the end of @foo
 
  $obj->logToFile(\&foo, "bar");
- 
+
 will call foo($obj, "bar")
 
 Note that your bulkmail object is the first argument passed to your function. It's not called as a method, but
@@ -1117,14 +1135,14 @@ the object is still the first argument passed.
 This is mainly used with attribues going through _file_accessor.
 
  package SomeClass;
- 
+
  SomeClass->add_attr(['FOO', '_file_accessor'], ">>");
  my $obj = SomeClass->new(
  	FOO => \&foo
  ) || die SomeClass->error();
- 
+
  my $val = $obj->logToFile($obj->FOO, "valid address);
- 
+
 Internally, logToFile calls convert_to_scalar on the value it is called with.
 
 This method is known to be able to return:
@@ -1136,11 +1154,11 @@ This method is known to be able to return:
 sub logToFile {
 	my $self	= shift;
 	my $file	= shift || return $self->error("Cannot log to file w/o file", "MB047");
-	
+
 	my $value	= shift;
-	
+
 	$value 		= $self->convert_to_scalar($value);
-	
+
 	if (ref $file eq "GLOB"){
 		print $file $value, "\015\012" if $value;
 		return 1;
@@ -1156,7 +1174,7 @@ sub logToFile {
 	else {
 		return $self->error("Cannot log to file...don't know what a $file is", "MBO005");
 	};
-	
+
 };
 
 =pod
@@ -1176,11 +1194,11 @@ This is useful to subclass if you ever want to log values other than simple scal
 sub convert_to_scalar {
 	my $self 	= shift;
 	my $value	= shift;
-	
+
 	my $v2 = ref $value ? $$value : $value;
-	
+
 	$v2 =~ s/[\015\012]//g if defined $v2;
-		
+
 	return $v2;
 };
 
@@ -1224,21 +1242,21 @@ ERRFILE may be either a coderef, globref, arrayref, or string literal.
 If a string literal, then Mail::Bulkmail::Object will attempt to open that file (in append mode) as your log:
 
  $bulk->ERRFILE("/path/to/my/error.file");
- 
+
 If a globref, it is assumed to be an open filehandle in append mode:
 
  open (E, ">>/path/to/my/error.file");
  $bulk->ERRFILE(\*E);
- 
+
 if a coderef, it is assumed to be a function to call with the address as an argument:
 
  sub E { print "ERROR : ", shift, "\n"};	#or whatever your code is
  $bulk->ERRFILE(\&E);
- 
+
 if an arrayref, then bad addresses will be pushed on to the end of it
 
  $bulk->ERRFILE(\@errors);
- 
+
 Use whichever item is most convenient, and Mail::Bulkmail::Object will take it from there.
 
 It is recommended you turn on ERRFILE in a debugging envrionment, and leave it off in production. You probably shouldn't
@@ -1290,7 +1308,7 @@ is stripped.
  so these are all the same:
  ERRFILE = /path/to/err.file
     ERRFILE        =     /path/to/err.file
-            ERRFILE =        /path/to/err.file            
+            ERRFILE =        /path/to/err.file
             										^^^^^extra spaces
 
 Your conf file is read by read_conf_file. As you saw in the docs for read_conf_file, it creates a hashref. The top
@@ -1300,9 +1318,9 @@ to define which package you're looking at.
  define package SomeClass
 
  define package OtherClass
- 
+
  ERRFILE = /path/to/err.file
- 
+
 So ERRFILE is now defined for OtherClass, but not for SomeClass (unless of course, OtherClass is a sub-class of
 SomeClass)
 
@@ -1311,27 +1329,27 @@ If you do not define a package, then the default package is assumed.
 Multiple entries in a conf file take the last one.
 
  define package SomeClass
- 
+
  ERRFILE = /path/to/err.file
  ERRFILE = /path/to/err.other.file
- 
+
 so SomeClass->ERRFILE is /path/to/err.other.file There is no way to programmatically access /path/to/err.file, the
 value was destroyed, even though it is still in the conf file.
 
 There is one magic value token...undef
 
  ERRFILE = undef
- 
+
 This will set ERRFILE to the perl value 'undef', as opposed to the literal string "undef"
 
 Sometimes, you will want to give a conf entry multiple values. Then, use the @= syntax.
 
  define package SomeClass
- 
+
  foo = 7
  bar @= 8
  bar @= 9
- 
+
 SomeClass->foo will be 7, SomeClass->bar will be [8, 9]
 
 There is no way to assign a value more complex than a scalar or an arrayref.
@@ -1343,38 +1361,38 @@ Comments are lines that begin with a #
 
  #connections stores the maximum number of connections we want
  connections = 7
- 
+
 
 If you want to get *really* fancy, you can restrict values to the user that is running the script. Use
 the :ID syntax for that.
 
  define package SomeClass
- 
+
  #user 87 gets this value
  87:foo	= 9
- 
+
  #user 93 gets this value
  93:foo = 10
- 
+
  #everyone else gets this value
  foo = 11
- 
+
 =head1 SAMPLE CONF FILE
 
  #this is in the default package
  ERRFILE = /path/to/err.file
- 
+
  define package Mail::Bulkmail::Server
  #set our Smtp Server
  Smtp	= your.smtp.cpm
- 
+
  #set our Port
  Port	= 25
- 
+
  define package JIM::SubClass
- 
+
  #store the IDs of the server objects we want to use by default
- 
+
  servers @= 7
  servers @= 19
  servers @= 34
@@ -1384,33 +1402,33 @@ the :ID syntax for that.
 In fact, we'll even get fancy, and specify an ABNF grammar for the conf file.
 
 	CONFFILE = *(LINE "\n")					; a conf file consists of 0 or more lines
-	
+
 	LINE = (
 			DEFINE 			; definition line
 			/ COMMENT 		; comment line
 			/ EQUATION 		; equation line
 			/ *(WSP)		; blank line
 		) "\n"				; followed by a newline character
-	
+
 	DEFINE = %b100 %b101 %b102 %b105 %b110 %b101 %b32 %b112 %b97 %b99 %b107 %b97 %b103 %b101 TEXT
 		; the literal string "define package" in lower case, followed by TEXT
-			
+
 	COMMENT = *(WSP) "#" TEXT
-	
+
 	EQUATION = *(WSP) (VARIABLE / USER_VARIABLE) *(WSP) EQUATION_SYMBOL *(WSP) VALUE *(WSP)
-	
+
 	USER_VARIABLE = USER *(WSP) ":" *(WSP) VARIABLE
-	
+
 	USER = 1*(DIGIT)
-	
+
 	EQUATION_SYMBOL = "=" / "@="
-	
+
 	VALUE = *(TEXT)
-	
+
 	USER_VARIABLE = *(TEXT)
-	
+
 	TEXT = VISIBLE *(VISIBLE / WSP) [VISIBLE]
-	
+
 	VISIBLE = %d33-%d126	; visible ascii characters
 
 
@@ -1422,7 +1440,7 @@ Mail::Bulkmail, Mail::Bulkmail::Server
 
 Copyright and (c) 1999, 2000, 2001, 2002, 2003 James A Thomason III (jim@jimandkoka.com). All rights reserved.
 Mail::Bulkmail::Object is is licensed under the artistic license for the creation classes related to Mail::Bulkmail, or
-to enable the use and running of those classes. 
+to enable the use and running of those classes.
 
 =head1 CONTACT INFO
 
